"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const events_1 = tslib_1.__importDefault(require("events"));
exports.eventEmitter = new events_1.default.EventEmitter();
// Emit event with better message error
function eventError(err, file) {
    if (err) {
        switch (err.code) {
            case 'EMFILE':
                exports.eventEmitter.emit('warn', file, `Too many open files, cannot ${err.syscall || 'access'}: `);
                break;
            case 'ENOENT':
                exports.eventEmitter.emit('warn', file, 'This file no longer exists: ');
                break;
            case 'EPERM':
                exports.eventEmitter.emit('error', file, `Operation not permitted on this file (${err.syscall}): `);
                break;
            case 'ENOTEMPTY':
                // Work as intended
                break;
            default:
                if (err.message === '64bit files are not yet supported.') {
                    exports.eventEmitter.emit('error', file, '64bit files are not yet supported.');
                }
                else {
                    console.log(err);
                    throw err;
                }
            // break
        }
    }
}
exports.eventError = eventError;
exports.tree = {};
const debug = process.argv.includes('--debug');
if (debug) {
    exports.eventEmitter.on('start', (file) => console.log('Starting ' + file));
    exports.eventEmitter.on('done', (file) => console.log('Done ' + file));
    exports.eventEmitter.on('verbose', (file, info) => console.log(info + file));
    exports.eventEmitter.on('warn', (file, err) => console.log(err + file));
    exports.eventEmitter.on('error', (file, err) => console.log(err + file));
}
