"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const util_1 = tslib_1.__importDefault(require("util"));
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const events_1 = require("./events");
class RmDir {
    constructor() {
        this.steps = [];
        this.compile = (uuid) => {
            events_1.tree[uuid] = [];
            return Object.assign((p, options, callback) => {
                if (callback === undefined && typeof options === 'function') {
                    callback = options;
                    options = undefined;
                }
                this.steps.reduce((prev, next) => {
                    return prev.then((p) => next(p, uuid));
                    /* .catch((err: NodeJS.ErrnoException) => {
                      eventError(err, p as string)
                      callback!(err)
                      return Promise.reject(err)
                    }) */
                }, this.init(p))
                    .then(() => callback(null))
                    .catch((err) => {
                    events_1.eventError(err, p);
                    callback(err);
                });
            }, { __promisify__: util_1.default.promisify(fs_1.default.rmdir) } // FIXME
            );
        };
    }
    init(p) {
        return new Promise((resolve) => {
            events_1.eventEmitter.emit('start', p);
            resolve(p);
        });
    }
    then(fun) {
        this.steps.push(fun);
        return this;
    }
    log() {
        this.steps.push(function (p, uuid) {
            return new Promise((resolve, reject) => {
                if (events_1.tree[uuid].includes(p)) {
                    resolve(p);
                }
                else {
                    events_1.tree[uuid].push(p);
                    /* const split = p.split(path.sep)
                    console.log('┃ '.repeat(split.length - 1) + '┠─' + split[split.length - 1]) */
                    const err = new Error('Log error (or is it?)');
                    err.code = 'ENOTEMPTY';
                    reject(err);
                }
            });
        });
        return this;
    }
    // Rename to random string
    rename() {
        this.steps.push(function (p) {
            return new Promise((resolve, reject) => {
                const newName = crypto_1.default.randomBytes(9).toString('base64').replace(/\//g, '0').replace(/\+/g, 'a');
                const newPath = path_1.default.join(path_1.default.dirname(p), newName);
                events_1.eventEmitter.emit('verbose', p, `Renaming to ${newName} `);
                fs_1.default.rename(p, newPath, (err) => {
                    if (err)
                        reject(err);
                    else
                        resolve(newPath);
                });
            });
        });
        return this;
    }
    // End function: remove the directory
    rmdir() {
        this.steps.push(function (p) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', p, 'Removing');
                fs_1.default.rmdir(p, (err) => {
                    if (err)
                        reject(err);
                    else {
                        events_1.eventEmitter.emit('done', p);
                        resolve(p);
                    }
                });
            });
        });
        return this.compile;
    }
}
exports.default = RmDir;
