"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const rimraf_1 = tslib_1.__importDefault(require("rimraf"));
const standards_1 = require("./standards"); // eslint-disable-line no-unused-vars
const events_1 = require("./events");
// to get the correct tree for each call
function getUUID() {
    return crypto_1.default.randomBytes(60).toString('base64').replace(/\//g, '0').replace(/\+/g, 'a');
}
function remove(path, options, callback) {
    const uuid = getUUID();
    // Parse if callback is provided
    if (callback === undefined && typeof options === 'function') {
        callback = options;
        options = { standard: 'secure' };
    }
    // Define standard if none is provided
    if (options === undefined)
        options = { standard: 'secure' };
    if (options.standard === undefined)
        options.standard = 'secure';
    if (callback)
        removeCallback(path, options, uuid, (err, goodTree) => callback(err, goodTree));
    else
        return removePromise(path, options, uuid);
}
exports.remove = remove;
// (module).exports = secureRm
const defaultGlobOpts = {
    nosort: true,
    silent: true
};
// Callback version
function removeCallback(path, options, uuid, callback) {
    if (options.customStandard) {
        rimraf_1.default(path, {
            unlink: options.customStandard.unlinkStandard(uuid),
            rmdir: options.customStandard.rmdirStandard(uuid),
            maxBusyTries: options.maxBusyTries || 3,
            emfileWait: options.emfileWait || 1000,
            disableGlob: options.glob === false ? true : options.disableGlob || false,
            glob: options.glob || defaultGlobOpts
        }, (err) => callback(err, events_1.tree[uuid]));
    }
    else if (standards_1.validIDs.includes(options.standard)) {
        rimraf_1.default(path, {
            unlink: standards_1.standards[options.standard].unlinkStandard(uuid),
            rmdir: standards_1.standards[options.standard].rmdirStandard(uuid),
            maxBusyTries: options.maxBusyTries || 3,
            emfileWait: options.emfileWait || 1000,
            disableGlob: options.glob === false ? true : options.disableGlob || false,
            glob: options.glob || defaultGlobOpts
        }, (err) => callback(err, events_1.tree[uuid]));
    }
    else {
        callback(new Error(`'${options.standard}' is not a valid ID. \nList of valid IDs: ${standards_1.validIDs}`), ['none']);
    }
}
// Promise version
function removePromise(path, options, uuid) {
    return new Promise((resolve, reject) => {
        if (options.customStandard) {
            rimraf_1.default(path, {
                unlink: options.customStandard.unlinkStandard(uuid),
                rmdir: options.customStandard.rmdirStandard(uuid),
                maxBusyTries: options.maxBusyTries || 3,
                emfileWait: options.emfileWait || 1000,
                disableGlob: options.glob === false ? true : options.disableGlob || false,
                glob: options.glob || defaultGlobOpts
            }, (err) => {
                if (err)
                    reject(err);
                else
                    resolve(events_1.tree[uuid]);
            });
        }
        else if (standards_1.validIDs.includes(options.standard)) {
            rimraf_1.default(path, {
                unlink: standards_1.standards[options.standard].unlinkStandard(uuid),
                rmdir: standards_1.standards[options.standard].rmdirStandard(uuid),
                maxBusyTries: options.maxBusyTries || 3,
                emfileWait: options.emfileWait || 1000,
                disableGlob: options.glob === false ? true : options.disableGlob || false,
                glob: options.glob || defaultGlobOpts
            }, (err) => {
                if (err)
                    reject(err);
                else
                    resolve(events_1.tree[uuid]);
            });
        }
        else {
            reject(new Error(`'${options.standard}' is not a valid ID. \nList of valid IDs: ${standards_1.validIDs}`));
        }
    });
}
