"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const util_1 = tslib_1.__importDefault(require("util"));
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const buffer_1 = require("buffer");
const events_1 = require("./events");
// Function to offset an array
// const offset = (arr: number[], offset: number) => [...arr.slice(offset), ...arr.slice(0, offset)]
class Unlink {
    constructor() {
        this.steps = [];
        this.compile = (uuid) => {
            events_1.tree[uuid] = [];
            return Object.assign((file, callback) => {
                this.steps.reduce((prev, next) => {
                    return prev.then(({ file, fileSize }) => next(file, fileSize, uuid));
                    /* .catch((err: NodeJS.ErrnoException) => {
                      eventError(err, file as string)
                      callback(err)
                      return Promise.reject(err)
                    }) */
                }, this.init(file))
                    .then(() => callback(null))
                    .catch((err) => {
                    events_1.eventError(err, file);
                    callback(err);
                });
            }, { __promisify__: util_1.default.promisify(fs_1.default.unlink) } // FIXME
            );
        };
    }
    init(file) {
        return new Promise((resolve, reject) => {
            fs_1.default.stat(file, (err, stats) => {
                if (err)
                    reject(err);
                else if (stats.size <= buffer_1.kMaxLength) {
                    events_1.eventEmitter.emit('start', file);
                    resolve({ file, fileSize: stats.size });
                }
                else {
                    reject(Error('64bit files are not yet supported.'));
                }
            });
        });
    }
    log() {
        this.steps.push(function (file, fileSize, uuid) {
            return new Promise((resolve) => {
                if (!events_1.tree[uuid].includes(file)) {
                    events_1.tree[uuid].push(file);
                }
                // const split = file.split(path.sep)
                // console.log('┃ '.repeat(split.length - 1) + '┠─' + split[split.length - 1])
                resolve({ file, fileSize });
            });
        });
        return this;
    }
    then(fun) {
        this.steps.push(fun);
        return this;
    }
    random(passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                const buffer = Buffer.alloc(fileSize);
                crypto_1.default.randomFill(buffer, (err, randomBuffer) => {
                    if (err)
                        reject(err);
                    events_1.eventEmitter.emit('verbose', file, 'Writing random data ');
                    fs_1.default.writeFile(file, randomBuffer, (err) => {
                        if (err)
                            reject(err);
                        resolve({ file, fileSize });
                    });
                });
            });
        }));
        return this;
    }
    zeroes(passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Writing zeroes ');
                fs_1.default.writeFile(file, Buffer.alloc(fileSize, 0b00000000), (err) => {
                    if (err)
                        reject(err);
                    resolve({ file, fileSize });
                });
            });
        }));
        return this;
    }
    ones(passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Writing ones ');
                fs_1.default.writeFile(file, Buffer.alloc(fileSize, 0b11111111), (err) => {
                    if (err)
                        reject(err);
                    resolve({ file, fileSize });
                });
            });
        }));
        return this;
    }
    // Write one byte on the whole file
    byte(data, passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, `Writing 0x${data.toString(16)} `);
                fs_1.default.writeFile(file, Buffer.alloc(fileSize, data), (err) => {
                    if (err)
                        reject(err);
                    resolve({ file, fileSize });
                });
            });
        }));
        return this;
    }
    // Write an array of bytes on the whole file
    byteArray(dataArray, passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, `Writing ${dataArray.map(x => '0x' + x.toString(16))} `);
                const dataConverted = Buffer.from(dataArray);
                fs_1.default.writeFile(file, Buffer.alloc(fileSize, dataConverted), (err) => {
                    if (err)
                        reject(err);
                    resolve({ file, fileSize });
                });
            });
        }));
        return this;
    }
    // A for loop
    forByte({ init, condition, increment }) {
        let i = init;
        while (true) {
            if (!condition(i)) {
                break;
            }
            const j = i; // Fix the value (the following function could be called after the increment)
            this.steps.push(function (file, fileSize) {
                return new Promise((resolve, reject) => {
                    events_1.eventEmitter.emit('verbose', file, `Writing 0x${j.toString(16)} `);
                    fs_1.default.writeFile(file, Buffer.alloc(fileSize, j), (err) => {
                        if (err)
                            reject(err);
                        resolve({ file, fileSize });
                    });
                });
            });
            i = increment(i);
        }
        return this;
    }
    randomByte(passes = 1) {
        this.steps = this.steps.concat(Array(passes).fill(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Writing random byte ');
                fs_1.default.writeFile(file, Buffer.alloc(fileSize, crypto_1.default.randomBytes(1)[0]), (err) => {
                    if (err)
                        reject(err);
                    resolve({ file, fileSize });
                });
            });
        }));
        return this;
    }
    // Write the binary complement
    complementary() {
        this.steps.push(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Reading file ');
                fs_1.default.readFile(file, (err, data) => {
                    if (err)
                        reject(err);
                    else {
                        events_1.eventEmitter.emit('verbose', file, 'Getting binary complement ');
                        for (let i = 0, l = fileSize; i < l; i++) {
                            data[i] = ~data[i];
                        }
                        events_1.eventEmitter.emit('verbose', file, 'Writing binary complement ');
                        fs_1.default.writeFile(file, data, (err) => {
                            if (err)
                                reject(err);
                            else
                                resolve({ file, fileSize });
                        });
                    }
                });
            });
        });
        return this;
    }
    // Rename to random string
    rename() {
        this.steps.push(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                const newName = crypto_1.default.randomBytes(9).toString('base64').replace(/\//g, '0').replace(/\+/g, 'a');
                const newPath = path_1.default.join(path_1.default.dirname(file), newName);
                events_1.eventEmitter.emit('verbose', file, `Renaming to ${newName} `);
                fs_1.default.rename(file, newPath, (err) => {
                    if (err)
                        reject(err);
                    else
                        resolve({ file: newPath, fileSize });
                });
            });
        });
        return this;
    }
    // Truncate to between 25% and 75% of the file size
    truncate() {
        this.steps.push(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Truncating ');
                const newSize = Math.floor((Math.random() * 0.5 + 0.25) * fileSize);
                fs_1.default.truncate(file, newSize, (err) => {
                    if (err)
                        reject(err);
                    else
                        resolve({ file, fileSize: newSize });
                });
            });
        });
        return this;
    }
    // End function: unlink the file pointer
    unlink() {
        this.steps.push(function (file, fileSize) {
            return new Promise((resolve, reject) => {
                events_1.eventEmitter.emit('verbose', file, 'Unlinking ');
                fs_1.default.unlink(file, (err) => {
                    if (err)
                        reject(err);
                    else {
                        events_1.eventEmitter.emit('done', file);
                        resolve({ file, fileSize });
                    }
                });
            });
        });
        return this.compile;
    }
}
exports.default = Unlink;
